@@ -1,37 +1,35 @@
 import pytz
 
-from datetime import datetime, timedelta
+from datetime import datetime
 from typing import Dict, Union, List, Optional, Iterable
 from functools import partial
 
 from .methods import all_methods, Method, ISNA
+from .exceptions import *
+from .enums import *
 
 __all__ = (
     "Data",
     "Date",
     "DateType",
     "Meta",
-    "LatitudeAdjustmentMethods",
-    "MidnightModes",
     "Timings",
     "Tune",
     "Prayer",
-    "Schools",
     "TimingsDateArg",
     "CalendarDateArg",
-    "DefaultArgs",
+    "Parameters",
     "Qibla",
     "Ism",
     "Method",
 )
-# TODO: docs (errors)
 
 
 class Tune:
     """
     Represents a Tune obj that is returned from API.
     Can be used to make an obj that will be used as a tune param in \
-    :class:`DefaultArgs`
+    :class:`Parameters`
 
     Attributes
     ----------
@@ -51,8 +49,22 @@ class Tune:
             The tune value for isha.
         midnight: :class:`int`
             The tune value for midnight.
+
+    *New in v1.0.: __iter__*
     """
 
+    __slots__ = (
+        "imsak",
+        "fajr",
+        "sunrise",
+        "dhuhr",
+        "asr",
+        "maghrib",
+        "sunset",
+        "isha",
+        "midnight",
+    )
+
     def __init__(
         self,
         Imsak: int = 0,
@@ -93,11 +105,21 @@ class Tune:
         -------
             :class:`Tune`
                 The created obj.
+
+        Raises
+        ------
+            :exc:`AsserationError`
+                Invalid string format.
         """
         args = s.split(",")
-        assert len(args) == 9, "Not valid string format"
+        assert (
+            len(args) == 9
+        ), "Invalid string format, must be in Tune.value format."
         return cls(*map(int, args))
 
+    def __iter__(self):
+        yield from map(int, self.value.split(","))
+
     def __repr__(self):  # pragma: no cover
         return "<Tune = {}>".format(self.value)
 
@@ -122,6 +144,8 @@ class Qibla:
     *New in v0.1.3*
     """
 
+    __slots__ = ("longitude", "latitude", "direction")
+
     def __init__(self, longitude: float, latitude: float, direction: float):
         self.longitude = longitude
         self.latitude = latitude
@@ -155,6 +179,8 @@ class Ism:
     *New in v0.1.3*
     """
 
+    __slots__ = ("name", "transliteration", "number", "en")
+
     def __init__(self, name: str, transliteration: str, number: int, en: dict):
         self.name = name
         self.transliteration = transliteration
@@ -168,28 +194,6 @@ class Ism:
         return hash(self.name)
 
 
-class Schools:
-    """Available schools"""
-
-    STANDARD = SHAFI = 0
-    HANAFI = 1
-
-
-class MidnightModes:
-    """Available midnight modes"""
-
-    STANDARD = 0
-    JAFARI = 1
-
-
-class LatitudeAdjustmentMethods:
-    """Available latitude adjustment methods"""
-
-    MIDDLE_OF_THE_NIGHT = 1
-    ONE_SEVENTH = 2
-    ANGLE_BASED = 3
-
-
 class Prayer:
     """Represents a Prayer obj.
 
@@ -212,6 +216,8 @@ class Prayer:
     *New in v0.1.2: timings, time_utc*
     """
 
+    __slots__ = ("timings", "name", "time", "time_utc", "str_time")
+
     def __init__(
         self, name: str, time: str, timings: "Timings", date: str = None
     ):
@@ -284,8 +290,15 @@ class CalendarDateArg:
 
         hijri: :class:`bool`
             Whether `year` given is a hijri year or not.
+
+    Raises
+    ------
+        :exc:`ValueError`
+            Month passed is not in 1-12 range.
     """
 
+    __slots__ = ("year", "month", "hijri", "annual")
+
     def __init__(
         self,
         year: int,
@@ -298,10 +311,10 @@ class CalendarDateArg:
                     "month argument expected to be in range 1-12"
                     " got {}".format(month)
                 )
-            self.month = month
+            self.month: int = month
             self.annual = "false"
         else:
-            self.month = 0
+            self.month: int = 0
             self.annual = "true"
 
         self.year = year
@@ -331,8 +344,14 @@ class TimingsDateArg:
         date: :class:`str`
             A date string in DD-MM-YYYY format.
 
+    Raises
+    ------
+        :exc:`AsserationError`
+            Invalid date string format.
     """
 
+    __slots__ = "date"
+
     def __init__(self, date: Optional[Union[str, int, datetime]] = None):
         if date is None:
             date = datetime.utcnow()
@@ -350,13 +369,13 @@ class TimingsDateArg:
                     "Expected DD-MM-YYYY date format got {!r} ".format(date)
                 )
 
-        self.date = date  # noqa
+        self.date: str = date  # noqa
 
     def __hash__(self):  # pragma: no cover
         return hash(self.date)
 
 
-class DefaultArgs:
+class Parameters:
     """
     Class to make an obj that will be used as a defaults param in getters.
 
@@ -364,7 +383,7 @@ class DefaultArgs:
     ----------
         method: :class:`methods.Method` or :class:`int`
             A prayer time calculation method, you can look into all methods \
-            from :meth:`AsyncClient.get_all_methods()`.
+            from :meth:`Client.get_all_methods()`.
             Default: ISNA (Islamic Society of North America).
 
         tune: Optional[:class:`Tune`]
@@ -385,6 +404,8 @@ class DefaultArgs:
             Example: Europe/London. Calculated using the
             co-ordinates provided by default. *This should be used only in
             getters that uses co-ordinates or it will be ignored.*
+            
+            *New in v0.2.*
 
         latitudeAdjustmentMethod: :class:`int`
             Method for adjusting times higher latitudes.
@@ -405,7 +426,9 @@ class DefaultArgs:
 
         method_params: Optional[dict[str, int or "null"]]
             Method's parameters. ``None`` if method wasn't custom.
-
+            
+            *New in v0.2.*
+        
         tune: :class:`str`
             Tune Value.
 
@@ -414,59 +437,97 @@ class DefaultArgs:
         midnightMode: :class:`int`
 
         timezonestring: :class:`str`
+            *New in v0.2.*
 
         latitudeAdjustmentMethod: :class:`int`
 
         adjustment: :class:`int`
+    
+    Raises
+    ------
+        :exc:`~aladhan.exceptions.InvalidMethod`
+            Method id passed was not in 0-15, 
+            Or passed an integer (99) instead of Method obj for a custom 
+            method use.
+            
+        :exc:`~aladhan.exceptions.InvalidTune`
+            Tune with a bad value was passed.
+            
+        :exc:`~aladhan.exceptions.InvalidSchool`
+            
+        :exc:`~aladhan.exceptions.InvalidMidnightMode`
 
-    *New in v0.2 method_params, timezonestring*
+        :exc:`~aladhan.exceptions.InvalidTimezone`
+        
+        :exc:`~aladhan.exceptions.InvalidLatAdjMethod`
+        
+        :exc:`~aladhan.exceptions.InvalidAdjustment`
     """
 
+    __slots__ = (
+        "method",
+        "method_params",
+        "tune",
+        "school",
+        "midnightMode",
+        "timezonestring",
+        "latitudeAdjustmentMethod",
+        "adjustment",
+    )
+
     def __init__(
         self,
         method: Union[Method, int] = ISNA,
         tune: Optional[Tune] = None,
-        school: int = Schools.SHAFI,
-        midnightMode: int = MidnightModes.STANDARD,  # noqa
+        school: Union[int, Schools] = Schools.SHAFI,
+        midnightMode: Union[  # noqa
+            int, MidnightModes
+        ] = MidnightModes.STANDARD,
         timezonestring: Optional[str] = None,
-        latitudeAdjustmentMethod: int = LatitudeAdjustmentMethods.ANGLE_BASED,  # noqa
+        latitudeAdjustmentMethod: Union[  # noqa
+            int, LatitudeAdjustmentMethods
+        ] = LatitudeAdjustmentMethods.ANGLE_BASED,
         adjustment: int = 0,
     ):
         # method
         self.method_params = None
         if isinstance(method, Method):
             if method.id == 99:
-                try:
-                    self.method_params = "{fajr},{maghrib},{isha}".format(
-                        **method.params
-                    )
-                except KeyError as e:
-                    raise KeyError(
-                        f"{e.args[0]!r} is required key in Method's params. "
-                        "https://aladhanpy.readthedocs.io/en/latest/"
-                        "api.html#aladhan.methods.Method for more info"
-                    )
+                self.method_params = method.params_str
             method = method.id
         elif method == 99:
-            raise TypeError(
-                "Pass Method object instead if you want to use custom method"
+            raise InvalidMethod(
+                "Pass Method object instead if you want to use custom method."
+                " for more info https://aladhanpy.readthedocs.io/en/latest/api.html#aladhan.methods.Method"
             )
         if method not in range(16) and method != 99:  # pragma: no cover
-            raise ValueError(
+            raise InvalidMethod(
                 "Expected method in 0-15 range or 99 got {!r}".format(method)
             )
-        self.method = method
+        self.method: int = method
 
         # tune
         if tune is None:
             tune = Tune().value
         elif isinstance(tune, Tune):
             tune = tune.value
-        self.tune = tune
+            ts = tune.split(",")
+            if len(ts) != 9 or not all(x.isdigit() for x in ts):
+                raise InvalidTune(
+                    "Invalid tune argument was passed. (tune.value = %s)" % tune
+                )
+        else:
+            raise InvalidTune(
+                "'tune' argument must be `Tune` object."
+                " got `%s` instead." % type(tune).__name__
+            )
+        self.tune: str = tune
 
         # school
+        if isinstance(school, Schools):
+            school = school.value
         if school not in (0, 1):  # pragma: no cover
-            raise ValueError(
+            raise InvalidSchool(
                 "School argument can only be either 0 or 1 got {!r}".format(
                     school
                 )
@@ -474,8 +535,10 @@ class DefaultArgs:
         self.school = school
 
         # midnight mode
+        if isinstance(midnightMode, MidnightModes):
+            midnightMode = midnightMode.value
         if midnightMode not in (0, 1):  # pragma: no cover
-            raise ValueError(
+            raise InvalidMidnightMode(
                 "midnightMode argument can only be either 0 or 1"
                 " got {!r}".format(midnightMode)
             )
@@ -483,18 +546,28 @@ class DefaultArgs:
 
         # timezone string
         if timezonestring and timezonestring not in pytz.all_timezones_set:
-            raise ValueError("Invalid timezone.")
-        self.timezonestring = timezonestring
+            raise InvalidTimezone(
+                "Invalid timezone ({!r}).".format(timezonestring)
+                + " https://www.php.net/manual/en/timezones.php for valid timezones."
+            )
+        self.timezonestring: str = timezonestring
 
         # lat adj methods
+        if isinstance(latitudeAdjustmentMethod, LatitudeAdjustmentMethods):
+            latitudeAdjustmentMethod = latitudeAdjustmentMethod.value
         if latitudeAdjustmentMethod not in (1, 2, 3):  # pragma: no cover
-            raise ValueError(
+            raise InvalidLatAdjMethod(
                 "latitudeAdjustmentMethod argument can only be either 1, 2 or 3"
                 " got {!r}".format(latitudeAdjustmentMethod)
             )
         self.latitudeAdjustmentMethod = latitudeAdjustmentMethod
 
         # adj
+        if not isinstance(adjustment, int):
+            raise InvalidAdjustment(
+                "Expected adjustment argument to be `int` got"
+                " `{}`".format(type(adjustment).__name__)
+            )
         self.adjustment = adjustment
 
     @property
@@ -549,6 +622,18 @@ class Meta:
             Used offset to tune timings.
     """
 
+    __slots__ = (
+        "data",
+        "longitude",
+        "latitude",
+        "timezone",
+        "method",
+        "latitudeAdjustmentMethod",
+        "midnightMode",
+        "school",
+        "offset",
+    )
+
     def __init__(
         self,
         data: "Data",
@@ -569,17 +654,17 @@ class Meta:
         self.latitudeAdjustmentMethod = latitudeAdjustmentMethod
         self.midnightMode = midnightMode
         self.school = school
-        self.offset = Tune(*offset)
+        self.offset = Tune(**offset)
 
     @property
-    def default_args(self):
-        """:class:`DefaultArgs`: returns a default args obj
+    def parameters(self):
+        """:class:`Parameters`: returns a Parameters obj
 
         .. warning::
             This will set method to ISNA if method was custom, and it will
             always set adjustment to 0.
         """
-        return DefaultArgs(  # TODO: testing
+        return Parameters(  # TODO: testing
             self.method or 2,
             self.offset,
             getattr(Schools, self.school.upper()),
@@ -650,6 +735,18 @@ class DateType:
             A list of holidays might be empty for hijri, always None for gregorian.
     """
 
+    __slots__ = (
+        "name",
+        "date",
+        "format",
+        "day",
+        "weekday",
+        "month",
+        "year",
+        "designation",
+        "holidays",
+    )
+
     def __init__(
         self,
         name: str,
@@ -705,6 +802,8 @@ class Date:
             Hijri date.
     """
 
+    __slots__ = ("data", "readable", "timestamp", "gregorian", "hijri")
+
     def __init__(
         self,
         data: "Data",
@@ -766,6 +865,19 @@ class Timings:
     *New in v0.1.4: __iter__*
     """
 
+    __slots__ = (
+        "data",
+        "imsak",
+        "fajr",
+        "sunrise",
+        "dhuhr",
+        "asr",
+        "maghrib",
+        "sunset",
+        "isha",
+        "midnight",
+    )
+
     def __init__(
         self,
         data: "Data",
@@ -819,15 +931,18 @@ class Timings:
             "Isha": self.isha,
         }
 
-    async def next_prayer(self) -> Prayer:
+    def next_prayer(self) -> Optional[Prayer]:
         """
         Get the next upcoming prayer.
-        Don't use this for old dates.
+        Returns ``None`` if the upcoming wasn't in the date,
+        so this will return ``None`` if it was from an old date.
 
         Returns
         -------
-            :class:`Prayer`
+            Optional[:class:`Prayer`]
                 The upcoming prayer.
+
+        *Changed in v1.0.: Returns None instead of recursive calls, and no longer awaitable.*
         """
         meta = self.data.meta
         now = datetime.utcnow()
@@ -836,19 +951,7 @@ class Timings:
             if now < val.time:
                 return val
 
-        return await (
-            await self.data.client.get_timings(
-                meta.longitude,
-                meta.latitude,
-                TimingsDateArg(
-                    datetime(
-                        val.time.year, val.time.month, val.time.day  # noqa
-                    )
-                    + timedelta(1)
-                ),
-                meta.default_args,
-            )
-        ).next_prayer()
+        return None
 
     def __iter__(self) -> Iterable[Prayer]:
         yield from self.as_dict.values()
@@ -880,10 +983,12 @@ class Data:
         timings: :class:`Timings`
             Represents the timings part.
 
-        client: :class:`AsyncClient`
+        client: :class:`Client`
             Represents the client that the Data were fetched from.
     """
 
+    __slots__ = ("meta", "date", "timings", "client")
+
     def __init__(self, timings: dict, date: dict, meta: dict, client):
         self.meta = Meta(self, **meta)
         self.date = Date(self, **date)
